#! /bin/bash
############################################################################
##                                                                        ##
##     Copyright (C) 2010-2011  YannUbuntu (yannubuntu@gmail.com)         ##
##                                                                        ##
## This program is free software: you can redistribute it and/or modify   ##
## it under the terms of the GNU General Public License as published by   ##
## the Free Software Foundation, either version 3 of the License, or      ##
## (at your option) any later version.                                    ##
##                                                                        ##
## This program is distributed in the hope that it will be useful,        ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of         ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          ##
## GNU General Public License for more details.                           ##
##                                                                        ##
## You should have received a copy of the GNU General Public License      ##
## along with this program.  If not, see <http://www.gnu.org/licenses/>.  ##
##                                                                        ##
############################################################################
## Special thanks to Babdu89, Josepe36, Meierfra & Gert Hulselmans. 
## ACTION LIBRAIRIES SPECIFIC TO BOOT-REPAIR

########################## REPAIR SEQUENCE DEPENDING ON USER CHOICE ##########################################
bootrepairactions() {
echo "
************************Actions"; debug_echo_important_variables
[[ "$FSCK_ACTION" = "yes" ]] && fsck_function					#Unmount all OS partition then remounts them
if [[ "$MBR_ACTION" != "nombraction" ]];then
	repair_boot_ini
	freed_space_function			#Requires Linux partitions to be mounted
fi
[[ "$BOOTFLAG_ACTION" = "setflag" ]] && [[ "$BOOTFLAG_TO_USE" ]] &&	bootflag_action $BOOTFLAG_TO_USE
if [[ "$MBR_ACTION" = "purge" ]];then
	check_internet_connection
	if [[ "$INTERNET" = "no-internet" ]];then
		end_of_pulsate
		zenity --error --title="$(eval_gettext "$CLEANNAME")" --text="$No_internet_connection_detected"
		echo 'SET@_mainwindow.show()'
	else
		purge_grub
	fi
else
	[[ "$UNHIDEBOOT_ACTION" = "yes" ]] && unhide_boot_menus_etc_default_grub	#Requires all OS partitions to be mounted
	if [[ "$MBR_ACTION" = "reinstall" ]];then
		reinstall_action
	elif [[ "$MBR_ACTION" = "restore" ]]; then
		restore_mbr
	fi
	unmount_all_and_success_repair
fi
}

########################## UNMOUNT ALL AND SUCCESS REPAIR ##########################################
unmount_all_and_success_repair() {
if [[ "$UNHIDEBOOT_ACTION" = "yes" ]];then
	unhide_boot_menus_xp
	unhide_boot_menus_grubcfg	#To replace the "-1"
fi
[[ "$PASTEBIN_ACTION" = "yes" ]] && pastebinaction
[[ "$SENDSTATS" != "nostats" ]] && [[ "$INTERNET" = "connected" ]] && statsbr
save_log_on_disks
unmount_all_blkid_partitions_except_df
end_of_pulsate
if [[ "$PASTEBIN_ACTION" = "yes" ]];then
	if [[ "$PASTEBIN_URL" ]];then
		FINALTEXT="${Please_note_the_following_url}\\n   $PASTEBIN_URL\\n   ${Indicate_it_in_case_still_pb}\\n   \
		boot.repair@gmail.com \\n${or_to_your_favorite_support_forum}\\n"
	elif [[ -f ${LOGREP}/RESULTS.txt ]];then
		FILENAME="Boot-Info-Summary_${DATE}.txt"
		cp ${LOGREP}/RESULTS.txt ~/${FILENAME}
		if [[ "$(type -p leafpad)" ]];then	#to avoid opening in term
			leafpad ~/${FILENAME} &
		else
			xdg-open ~/${FILENAME} &
		fi
		sleep 2
		update_translations
		FINALTEXT="${FILENAME_has_been_created}\\n${Indicate_its_content_in_case_still_pb}\\n   boot.repair@gmail.com \\n${or_to_your_favorite_support_forum}\\n"	
	else
		zenity --error --title="$(eval_gettext "$CLEANNAME")" --text="Could not create BootInfoSummary. Please report this bug to boot.repair@gmail.com"
	fi
fi
if [[ "$MBR_ACTION" != "nombraction" ]] || [[ "$UNHIDEBOOT_ACTION" = "yes" ]] || [[ "$FSCK_ACTION" = "yes" ]] || [[ "$BOOTFLAG_ACTION" != "noflag" ]];then
	FINALTEXT="${Boot_sector_successfully_repaired}\\n${FINALTEXT}${You_can_now_reboot}"
else
	FINALTEXT="${FINALTEXT}${No_change_have_been_performed_on_your_computer_See_you_soon}"
fi
zenity --info --title="$(eval_gettext "$CLEANNAME")" --text="${FINALTEXT}"
rm -r $TMP_FOLDER_TO_BE_CLEARED
echo "[debug]End of unmount_all_and_success_repair"
echo 'EXIT@@'
}

########################################### REPAIR PARTITIONS (FSCK) ##################################################################
fsck_function() {
force_unmount_blkid_partitions
#fsck -fyM  # repair partitions detected in the /etc/fstab except those mounted
for ((i=1;i<=NBOFPARTITIONS;i++)); do
	echo "SET@_label0.set_text('''$Repair_file_systems ${LISTOFPARTITIONS[$i]}. $(eval_gettext $'This may require several minutes...')''')"
	temp="$(fdisk -l | grep "/dev/${LISTOFPARTITIONS[$i]} " )"
	if [[ "$(grep -i "NTFS" <<< "$temp" )" ]];then
		FUNCTION="NTFSFIX"
		PACKAGELIST="ntfsprogs"
		FILETOTEST="ntfsfix"
		installpackagelist
		[[ "$(type -p $FILETOTEST)" ]] && ntfsfix /dev/${LISTOFPARTITIONS[$i]}	#Repair NTFS partitions
	else
		fsck -fyM /dev/${LISTOFPARTITIONS[$i]}	#Repair other partitions
	fi
done
mount_all_blkid_partitions_except_df
}

#Called by fsck_function
force_unmount_blkid_partitions() {
end_of_pulsate
zenity --info --title="$(eval_gettext "$CLEANNAME")" --text="Filesystem repair requires to unmount partitions. \
Please close all your programs, then close this window to continue."
beginning_of_pulsate
echo "Force Unmount all blkid partitions (for fsck) except / /boot /cdrom /dev /etc /home /opt /pas /proc /rofs /sys /tmp /usr /var "
pkill pcmanfm	#To avoid it automounts
for ((i=1;i<=NBOFPARTITIONS;i++)); do
	if [[ "${BLKIDMNT_POINT[$i]}" ]] && [[ ! "${BLKIDMNT_POINT[$i]}" =~ /boot ]] && [[ "${BLKIDMNT_POINT[$i]}" != /cdrom ]] \
	&& [[ ! "${BLKIDMNT_POINT[$i]}" =~ /dev ]] && [[ ! "${BLKIDMNT_POINT[$i]}" =~ /etc ]] \
	&& [[ "${BLKIDMNT_POINT[$i]}" != /home ]] && [[ ! "${BLKIDMNT_POINT[$i]}" =~ /opt ]] \
	&& [[ ! "${BLKIDMNT_POINT[$i]}" =~ /pas ]] && [[ ! "${BLKIDMNT_POINT[$i]}" =~ /proc ]] \
	&& [[ ! "${BLKIDMNT_POINT[$i]}" =~ /rofs ]] && [[ ! "${BLKIDMNT_POINT[$i]}" =~ /sys ]] \
	&& [[ ! "${BLKIDMNT_POINT[$i]}" =~ /tmp ]] && [[ ! "${BLKIDMNT_POINT[$i]}" =~ /usr ]] \
	&& [[ ! "${BLKIDMNT_POINT[$i]}" =~ /var ]];then
		umount "${BLKIDMNT_POINT[$i]}"
	fi
done
}

########################################### FREED SPACE ACTION ##################################################################
freed_space_function() {
#Workaround for this bug: https://bugs.launchpad.net/bugs/610358
echo "[debug]Freed space function"
echo "SET@_label0.set_text('''Checking full partitions. $(eval_gettext $'This may require several minutes...')''')"
#for ((i=1;i<=QTY_OF_PART_WITH_APTGET;i++)); do 
#	if [[ "${OS_PARTITION[${LIST_OF_PART_WITH_APTGET[$i]}]}" = "$CURRENTSESSIONPARTITION" ]];then
#		apt-get -y clean
#		apt-get -y autoremove
#	else
#		for w in dev dev/pts proc sys; do
#			mount -B /$w "${MNT_PATH[${LIST_OF_PART_WITH_APTGET[$i]}]}/$w"
#		done  # /dev/pts ???
#		chroot "${MNT_PATH[${LIST_OF_PART_WITH_APTGET[$i]}]}" apt-get -y clean
#		chroot "${MNT_PATH[${LIST_OF_PART_WITH_APTGET[$i]}]}" apt-get -y autoremove
#		pkill pcmanfm	#To avoid it automounts
#		for w in dev/pts dev proc sys; do
#			umount "${MNT_PATH[${LIST_OF_PART_WITH_APTGET[$i]}]}/$w"
#		done
#	fi
#done
# Warn if still full partition
for ((i=1;i<=TOTAL_QUANTITY_OF_OS;i++)); do
	#echo "$(df /dev/${OS_PARTITION[$i]} | grep "/dev/" )"
	USEDPERCENT=$(df /dev/${OS_PARTITION[$i]} | grep "/" | grep "%" ); USEDPERCENT=${USEDPERCENT%%\%*}; USEDPERCENT=${USEDPERCENT##* }
	if [[ "$USEDPERCENT" -ge "95" ]];then
		echo "${OS_PARTITION[$i]} is $USEDPERCENT % full"
		end_of_pulsate
		if [[ -d "${MNT_PATH[$i]}/home" ]];then
			xdg-open "${MNT_PATH[$i]}/home" &
		elif [[ -d "${MNT_PATH[$i]}/Documents and Settings" ]];then
			xdg-open "${MNT_PATH[$i]}/Documents and Settings" &
		elif [[ "${OS_PARTITION[$i]}" = "$CURRENTSESSIONPARTITION" ]];then
			xdg-open "/" &
		elif [[ "${MNT_PATH[$i]}" =~ "/mnt/$PACK_NAME" ]];then #To avoid https://bugs.launchpad.net/ubuntu/+source/xdg-utils/+bug/821284
			xdg-open "/mnt/$PACK_NAME" &
		else
			xdg-open "/" &
		fi
		THISPARTITION="${OS_PARTITION[$i]} \(${OS_NAME[$i]}\)" #TODO: integrate variables into mo for arabic translation
		update_translations
		zenity --warning --title="$(eval_gettext "$CLEANNAME")" --text="${THISPARTITION_is_nearly_full} ${This_can_prevent_to_start_it}. \
		$Please_use_the_file_browser $Close_this_window_when_finished"
		USEDPERCENT=$(df /dev/${OS_PARTITION[$i]} | grep "/dev/" ); USEDPERCENT=${USEDPERCENT%%\%*}; USEDPERCENT=${USEDPERCENT##* }
		if [[ "$USEDPERCENT" -ge "97" ]];then
			zenity --warning --title="$(eval_gettext "$CLEANNAME")" --text="${THISPARTITION_is_still_full} ${This_can_prevent_to_start_it} (${Power_manager_error})."
		fi
		beginning_of_pulsate
	fi
done
}


######################### STATS FOR IMPROVING BOOT-REPAIR##################
statsbr() {
stats
$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/bi $URLST.$MAIN_MENU.$CODO
if [[ "$MAIN_MENU" != "bootinfo" ]];then
	$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/re $URLST.repair.$CODO
	$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/ma $URLST.$MBR_ACTION.$CODO
	[[ "$FSCK_ACTION" = "yes" ]] &&	$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/fsck $URLST.fsck.$CODO
	[[ "$UNHIDEBOOT_ACTION" = "yes" ]] && $WGETST ${TMP_FOLDER_TO_BE_CLEARED}/uh $URLST.unhide.$CODO
	[[ "$UNCOMMENT_GFXMODE" = "yes" ]] && $WGETST ${TMP_FOLDER_TO_BE_CLEARED}/gf $URLST.gfx.$CODO
	[[ "$ADD_KERNEL_OPTION" = "yes" ]] && $WGETST ${TMP_FOLDER_TO_BE_CLEARED}/ke $URLST.kernel.$CODO
	[[ "$ATA" = "ata" ]] && $WGETST ${TMP_FOLDER_TO_BE_CLEARED}/at $URLST.ata.$CODO
	[[ "$REMOVABLEDISK" = "yes" ]] && $WGETST ${TMP_FOLDER_TO_BE_CLEARED}/gf $URLST.removabledisk.$CODO
	[[ "$GRUBPACKAGE" = "grub-efi" ]] && $WGETST ${TMP_FOLDER_TO_BE_CLEARED}/efi $URLST.efi.$CODO
	if [[ "$(lsb_release -drcs)" =~ Ubuntu ]];then
		$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/ub $URLST.ubuntu.$CODO
	elif [[ "$(lsb_release -drcs)" =~ Debian ]];then
		$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/de $URLST.debian.$CODO
	elif [[ "$(lsb_release -drcs)" =~ Mint ]];then
		$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/gf $URLST.gfx.$CODO
	elif [[ "$(lsb_release -drcs)" =~ Hybryde ]];then
		$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/hy $URLST.hybryde.$CODO
	else
		$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/oh $URLST.otherhost.$CODO
	fi
	if [[ "$QUANTITY_OF_DETECTED_LINUX" != "0" ]] && [[ "$QUANTITY_OF_DETECTED_WINDOWS" = "0" ]] && [[ "$QUANTITY_OF_DETECTED_MACOS" = "0" ]] \
	&& [[ "$QUANTITY_OF_UNKNOWN_OS" = "0" ]];then
		$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/lo $URLST.linuxonly.$CODO
	elif [[ "$QUANTITY_OF_DETECTED_LINUX" = "0" ]] && [[ "$QUANTITY_OF_DETECTED_WINDOWS" != "0" ]] && [[ "$QUANTITY_OF_DETECTED_MACOS" = "0" ]] \
	&& [[ "$QUANTITY_OF_UNKNOWN_OS" = "0" ]];then
		$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/wo $URLST.winonly.$CODO
	elif [[ "$QUANTITY_OF_DETECTED_LINUX" = "0" ]] && [[ "$QUANTITY_OF_DETECTED_WINDOWS" = "0" ]] && [[ "$QUANTITY_OF_DETECTED_MACOS" != "0" ]] \
	&& [[ "$QUANTITY_OF_UNKNOWN_OS" = "0" ]];then
		$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/mo $URLST.maconly.$CODO
	fi
	[[ "$QUANTITY_OF_UNKNOWN_OS" != "0" ]] && $WGETST ${TMP_FOLDER_TO_BE_CLEARED}/uo $URLST.unknownos.$CODO
	if [[ "$TOTAL_QUANTITY_OF_OS" = "0" ]];then
		$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/0o $URLST.0os.$CODO
	elif [[ "$TOTAL_QUANTITY_OF_OS" = "1" ]];then
		$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/1o $URLST.1os.$CODO
	elif [[ "$TOTAL_QUANTITY_OF_OS" = "2" ]];then
		$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/2o $URLST.2os.$CODO
	else
		$WGETST ${TMP_FOLDER_TO_BE_CLEARED}/3o $URLST.3osormore.$CODO
	fi
	[[ "$BLKID" =~ LVM2_member ]] && $WGETST ${TMP_FOLDER_TO_BE_CLEARED}/lv $URLST.lvm.$CODO
	[[ "$DMRAID" ]] && $WGETST ${TMP_FOLDER_TO_BE_CLEARED}/dm $URLST.dmraid.$CODO
	[[ "$QTY_OF_DISKS_WITH_BACKUP" != "0" ]] && $WGETST ${TMP_FOLDER_TO_BE_CLEARED}/cu $URLST.cleanubiquity.$CODO
fi
}


########################## PASTEBIN ACTION ##########################################
pastebinaction() {
check_internet_connection
[[ "$INTERNET" != "connected" ]] && ask_internet_connection
if [[ "$INTERNET" = "connected" ]];then
	FUNCTION="BootInfo"
	PACKAGELIST=""
	for temp in pastebinit gawk lzma;do
		[[ ! "$(type -p $temp)" ]] && PACKAGELIST="$temp $PACKAGELIST"
	done
	FILETOTEST="pastebinit gawk lzma"
	installpackagelist
fi
echo "SET@_label0.set_text('''$Create_a_BootInfo_report. $(eval_gettext $'Please wait few seconds...')''')"
cp "$TMP_LOG.tee" "$TMP_LOG.pastetmp"
sed -i "/^SET@/ d" "$TMP_LOG.pastetmp"
sed -i "/^DEBUG=>/ d" "$TMP_LOG.pastetmp"
sed -i "/^\[debug\]/ d" "$TMP_LOG.pastetmp"
sed -i "/^COMBO@@/ d" "$TMP_LOG.pastetmp"
sed -i "/^done/ d" "$TMP_LOG.pastetmp"
sed -i "/^1+0/ d" "$TMP_LOG.pastetmp"
while read line; do
	if [[ ! "$line" =~ B\/s ]] && [[ ! "$line" =~ "while true" ]] && [[ ! "$line" =~ "sleep 0" ]];then #[[ ! "$line" =~ "gpg: " ]] && [[ ! "$line" =~ "Executing: gpg " ]]
		echo "$line" >> "$TMP_LOG.paste"
	fi
done < <(echo "$(< $TMP_LOG.pastetmp)")
rm "$TMP_LOG.pastetmp"
unmount_all_blkid_partitions_except_df # necessary ?
kill_nautilus &
#courtesy of forum members Meierfra & Gert Hulselmans
cp /usr/share/${PACK_NAME}/bis${BISGIT}.sh ${TMP_FOLDER_TO_BE_CLEARED}/bis${BISGIT}.sh
LC_MESSAGES=C bash ${TMP_FOLDER_TO_BE_CLEARED}/bis${BISGIT}.sh
echo "ADDITIONAL INFORMATION :" >> ${TMP_FOLDER_TO_BE_CLEARED}/RESULTS.txt
cat $TMP_LOG.paste >> ${TMP_FOLDER_TO_BE_CLEARED}/RESULTS.txt
cp ${TMP_FOLDER_TO_BE_CLEARED}/RESULTS.txt ${LOGREP}/
check_internet_connection
if [[ "$INTERNET" = "connected" ]] && [[ "$(type -p pastebinit)" ]] && [[ "$(type -p gawk)" ]];then
	if [[ "$(lsb_release -is)" != "Ubuntu" ]];then
		PASTEBIN_URL=$(cat ${LOGREP}/RESULTS.txt | pastebinit -a boot-repair -f bash -b http://paste.debian.net)
	fi
	if [[ "$(lsb_release -is)" = "Ubuntu" ]] || [[ "$PASTEBIN_URL" = "http://paste.debian.net/" ]];then #workaround for debianpaste bug
		PASTEBIN_URL=$(cat ${LOGREP}/RESULTS.txt | pastebinit -a boot-repair -f bash -b http://paste.ubuntu.com)
	fi
fi
mount_all_blkid_partitions_except_df #For logs
}
